@@ -17,7 +17,11 @@
 """
 import warnings
 
+from tornado.httpclient import AsyncHTTPClient
+
 import toml
+from inmanta.config import AuthJWTConfig
+from inmanta.logging import InmantaLoggerConfig
 
 """
 About the use of @parametrize_any and @slowtest:
@@ -80,8 +84,9 @@ import traceback
 import uuid
 import venv
 from collections import abc
+from collections.abc import AsyncIterator, Awaitable, Iterator
 from configparser import ConfigParser
-from typing import AsyncIterator, Awaitable, Callable, Dict, Iterator, List, Optional, Tuple, Union
+from typing import Callable, Dict, Optional, Union
 
 import asyncpg
 import pkg_resources
@@ -94,7 +99,6 @@ from click import testing
 from pkg_resources import Requirement
 from pyformance.registry import MetricsRegistry
 from tornado import netutil
-from tornado.platform.asyncio import AnyThreadEventLoopPolicy
 
 import build.env
 import inmanta
@@ -103,26 +107,30 @@ import inmanta.app
 import inmanta.compiler as compiler
 import inmanta.compiler.config
 import inmanta.main
+import inmanta.user_setup
 from inmanta import config, const, data, env, loader, protocol, resources
+from inmanta.agent import config as agent_cfg
 from inmanta.agent import handler
 from inmanta.agent.agent import Agent
 from inmanta.ast import CompilerException
 from inmanta.data.schema import SCHEMA_VERSION_TABLE
 from inmanta.db import util as db_util
-from inmanta.env import LocalPackagePath, VirtualEnv, mock_process_env
+from inmanta.env import CommandRunner, LocalPackagePath, VirtualEnv, mock_process_env
 from inmanta.export import ResourceDict, cfg_env, unknown_parameters
 from inmanta.module import InmantaModuleRequirement, InstallMode, Project, RelationPrecedenceRule
-from inmanta.moduletool import IsolatedEnvBuilderCached, ModuleTool, V2ModuleBuilder
+from inmanta.moduletool import DefaultIsolatedEnvCached, ModuleTool, V2ModuleBuilder
 from inmanta.parser.plyInmantaParser import cache_manager
 from inmanta.protocol import VersionMatch
 from inmanta.server import SLICE_AGENT_MANAGER, SLICE_COMPILER
 from inmanta.server.bootloader import InmantaBootloader
 from inmanta.server.protocol import Server, SliceStartupException
+from inmanta.server.services import orchestrationservice
 from inmanta.server.services.compilerservice import CompilerService, CompileRun
 from inmanta.types import JsonType
 from inmanta.warnings import WarningsManager
 from libpip2pi.commands import dir2pi
 from packaging.version import Version
+from pytest_postgresql import factories
 
 # Import test modules differently when conftest is put into the inmanta_tests packages
 PYTEST_PLUGIN_MODE: bool = __file__ and os.path.dirname(__file__).split("/")[-1] == "inmanta_tests"
@@ -145,6 +153,8 @@ TABLES_TO_KEEP = [x.table_name() for x in data._classes] + ["resourceaction_reso
 # before it's saved.
 initial_cwd = os.getcwd()
 
+pg_logfile = os.path.join(initial_cwd, "pg.log")
+
 
 def _pytest_configure_plugin_mode(config: "pytest.Config") -> None:
     # register custom markers
@@ -221,20 +231,53 @@ def pytest_runtest_setup(item: "pytest.Item"):
             pytest.skip("Skipping old migration test")
 
 
+# adds a custom log location for postgres
+postgresql_proc_with_log = factories.postgresql_proc(startparams=f"--log='{pg_logfile}'")
+
+
 @pytest.fixture(scope="session")
 def postgres_db(request: pytest.FixtureRequest):
     """This fixture loads the pytest-postgresql fixture. When --postgresql-host is set, it will use the noproc
     fixture to use an external database. Without this option, an "embedded" postgres is started.
     """
+
     option_name = "postgresql_host"
     conf = request.config.getoption(option_name)
     if conf:
         fixture = "postgresql_noproc"
     else:
-        fixture = "postgresql_proc"
+        fixture = "postgresql_proc_with_log"
 
     logger.info("Using database fixture %s", fixture)
-    yield request.getfixturevalue(fixture)
+    pg = request.getfixturevalue(fixture)
+    yield pg
+
+    if os.path.exists(pg_logfile):
+        has_deadlock = False
+        with open(pg_logfile) as fh:
+            for line in fh:
+                if "deadlock" in line:
+                    has_deadlock = True
+                    break
+            sublogger = logging.getLogger("pytest.postgresql.deadlock")
+            for line in fh:
+                sublogger.warning("%s", line)
+        os.remove(pg_logfile)
+        assert not has_deadlock
+
+
+@pytest.fixture
+async def run_without_keeping_psql_logs(postgres_db):
+    if os.path.exists(pg_logfile):
+        # Store the original content of the logfile
+        with open(pg_logfile) as file:
+            original_content = file.read()
+    yield
+
+    if os.path.exists(pg_logfile):
+        # Restore the original content of the logfile
+        with open(pg_logfile, "w") as file:
+            file.write(original_content)
 
 
 @pytest.fixture
@@ -379,7 +422,7 @@ async def clean_db(postgresql_pool, create_db, postgres_db):
 
 @pytest.fixture(scope="function")
 def get_columns_in_db_table(postgresql_client):
-    async def _get_columns_in_db_table(table_name: str) -> List[str]:
+    async def _get_columns_in_db_table(table_name: str) -> list[str]:
         result = await postgresql_client.fetch(
             "SELECT column_name "
             "FROM information_schema.columns "
@@ -392,7 +435,7 @@ def get_columns_in_db_table(postgresql_client):
 
 @pytest.fixture(scope="function")
 def get_primary_key_columns_in_db_table(postgresql_client):
-    async def _get_primary_key_columns_in_db_table(table_name: str) -> List[str]:
+    async def _get_primary_key_columns_in_db_table(table_name: str) -> list[str]:
         # Query taken from here: https://wiki.postgresql.org/wiki/Retrieve_primary_key_columns
         result = await postgresql_client.fetch(
             "SELECT a.attname FROM pg_index i "
@@ -407,7 +450,7 @@ def get_primary_key_columns_in_db_table(postgresql_client):
 
 @pytest.fixture(scope="function")
 def get_tables_in_db(postgresql_client):
-    async def _get_tables_in_db() -> List[str]:
+    async def _get_tables_in_db() -> list[str]:
         result = await postgresql_client.fetch("SELECT table_name FROM information_schema.tables WHERE table_schema='public'")
         return [r["table_name"] for r in result]
 
@@ -415,18 +458,41 @@ def get_tables_in_db(postgresql_client):
 
 
 @pytest.fixture(scope="function")
-def get_custom_postgresql_types(postgresql_client) -> Callable[[], Awaitable[List[str]]]:
+def get_custom_postgresql_types(postgresql_client) -> Callable[[], Awaitable[list[str]]]:
     """
     Fixture that returns an async callable that returns all the custom types defined
     in the PostgreSQL database.
     """
 
-    async def f() -> List[str]:
+    async def f() -> list[str]:
         return await db_util.postgres_get_custom_types(postgresql_client)
 
     return f
 
 
+@pytest.fixture(scope="function")
+def get_type_of_column(postgresql_client) -> Callable[[], Awaitable[Optional[str]]]:
+    """
+    Fixture that returns the type of a column in a table
+    """
+
+    async def _get_type_of_column(table_name: str, column_name: str) -> Optional[str]:
+        data_type = await postgresql_client.fetchval(
+            """
+                SELECT data_type
+                FROM information_schema.columns
+                WHERE table_schema = 'public'
+                    AND table_name = $1
+                    AND column_name = $2;
+            """,
+            table_name,
+            column_name,
+        )
+        return data_type
+
+    return _get_type_of_column
+
+
 @pytest.fixture(scope="function")
 def deactive_venv():
     old_os_path = os.environ.get("PATH", "")
@@ -496,7 +562,7 @@ def clean_reset_session():
     Execute cleanup tasks that should only run at the end of the test suite.
     """
     yield
-    IsolatedEnvBuilderCached.get_instance().destroy()
+    DefaultIsolatedEnvCached.get_instance().destroy()
 
 
 def reset_all_objects():
@@ -509,13 +575,16 @@ def reset_all_objects():
     Project._project = None
     unknown_parameters.clear()
     InmantaBootloader.AVAILABLE_EXTENSIONS = None
-    V2ModuleBuilder.DISABLE_ISOLATED_ENV_BUILDER_CACHE = False
+    V2ModuleBuilder.DISABLE_DEFAULT_ISOLATED_ENV_CACHED = False
     compiler.Finalizers.reset_finalizers()
+    AuthJWTConfig.reset()
+    InmantaLoggerConfig.clean_instance()
+    AsyncHTTPClient.configure(None)
 
 
 @pytest.fixture()
 def disable_isolated_env_builder_cache() -> None:
-    V2ModuleBuilder.DISABLE_ISOLATED_ENV_BUILDER_CACHE = True
+    V2ModuleBuilder.DISABLE_DEFAULT_ISOLATED_ENV_CACHED = True
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -528,11 +597,11 @@ def restore_cwd():
 
 
 @pytest.fixture(scope="function")
-def no_agent_backoff():
-    backoff = inmanta.agent.agent.GET_RESOURCE_BACKOFF
-    inmanta.agent.agent.GET_RESOURCE_BACKOFF = 0
+def no_agent_backoff(inmanta_config: ConfigParser) -> None:
+    old_backoff = agent_cfg.agent_get_resource_backoff.get()
+    inmanta_config.set(section="config", option="agent-get-resource-backoff", value="0")
     yield
-    inmanta.agent.agent.GET_RESOURCE_BACKOFF = backoff
+    inmanta_config.set(section="config", option="agent-get-resource-backoff", value=str(old_backoff))
 
 
 @pytest.fixture()
@@ -554,7 +623,7 @@ def inmanta_config() -> Iterator[ConfigParser]:
     config.Config.load_config()
     config.Config.set("auth_jwt_default", "algorithm", "HS256")
     config.Config.set("auth_jwt_default", "sign", "true")
-    config.Config.set("auth_jwt_default", "client_types", "agent,compiler")
+    config.Config.set("auth_jwt_default", "client_types", "agent,compiler,api")
     config.Config.set("auth_jwt_default", "key", "rID3kG4OwGpajIsxnGDhat4UFcMkyFZQc1y3oKQTPRs")
     config.Config.set("auth_jwt_default", "expire", "0")
     config.Config.set("auth_jwt_default", "issuer", "https://localhost:8888/")
@@ -612,10 +681,10 @@ async def agent_factory(server):
     async def create_agent(
         environment: uuid.UUID,
         hostname: Optional[str] = None,
-        agent_map: Optional[Dict[str, str]] = None,
+        agent_map: Optional[dict[str, str]] = None,
         code_loader: bool = False,
-        agent_names: List[str] = [],
-    ) -> None:
+        agent_names: list[str] = [],
+    ) -> Agent:
         a = Agent(hostname=hostname, environment=environment, agent_map=agent_map, code_loader=code_loader)
         for agent_name in agent_names:
             await a.add_end_point_name(agent_name)
@@ -629,16 +698,21 @@ async def agent_factory(server):
 
 
 @pytest.fixture(scope="function")
-async def autostarted_agent(server, environment):
+async def autostarted_agent(server, client, environment):
     """Configure agent1 as an autostarted agent."""
-    env = await data.Environment.get_by_id(uuid.UUID(environment))
-    await env.set(data.AUTOSTART_AGENT_MAP, {"internal": "", "agent1": ""})
-    await env.set(data.AUTO_DEPLOY, True)
-    await env.set(data.PUSH_ON_AUTO_DEPLOY, True)
+    result = await client.set_setting(environment, data.AUTOSTART_AGENT_MAP, {"internal": "", "agent1": ""})
+    assert result.code == 200
+    result = await client.set_setting(environment, data.AUTO_DEPLOY, True)
+    assert result.code == 200
+    result = await client.set_setting(environment, data.PUSH_ON_AUTO_DEPLOY, True)
+    assert result.code == 200
     # disable deploy and repair intervals
-    await env.set(data.AUTOSTART_AGENT_DEPLOY_INTERVAL, 0)
-    await env.set(data.AUTOSTART_AGENT_REPAIR_INTERVAL, 0)
-    await env.set(data.AUTOSTART_ON_START, True)
+    result = await client.set_setting(environment, data.AUTOSTART_AGENT_DEPLOY_INTERVAL, 0)
+    assert result.code == 200
+    result = await client.set_setting(environment, data.AUTOSTART_AGENT_REPAIR_INTERVAL, 0)
+    assert result.code == 200
+    result = await client.set_setting(environment, data.AUTOSTART_ON_START, True)
+    assert result.code == 200
 
 
 @pytest.fixture(scope="function")
@@ -958,7 +1032,7 @@ def write_db_update_file():
     yield _write_db_update_file
 
 
-class KeepOnFail(object):
+class KeepOnFail:
     def keep(self) -> "Optional[Dict[str, str]]":
         pass
 
@@ -981,15 +1055,7 @@ def pytest_runtest_makereport(item, call):
 
         if resources:
             # we are behind report formatting, so write to report, not item
-            rep.sections.append(
-                ("Resources Kept", "\n".join(["%s %s" % (label, resource) for label, resource in resources.items()]))
-            )
-
-
-async def off_main_thread(func):
-    # work around for https://github.com/pytest-dev/pytest-asyncio/issues/168
-    asyncio.set_event_loop_policy(AnyThreadEventLoopPolicy())
-    return await asyncio.get_event_loop().run_in_executor(None, func)
+            rep.sections.append(("Resources Kept", "\n".join([f"{label} {resource}" for label, resource in resources.items()])))
 
 
 class ReentrantVirtualEnv(VirtualEnv):
@@ -1002,7 +1068,7 @@ class ReentrantVirtualEnv(VirtualEnv):
     """
 
     def __init__(self, env_path: str) -> None:
-        super(ReentrantVirtualEnv, self).__init__(env_path)
+        super().__init__(env_path)
         self.working_set = None
 
     def deactivate(self):
@@ -1032,7 +1098,7 @@ class ReentrantVirtualEnv(VirtualEnv):
 class SnippetCompilationTest(KeepOnFail):
     def setUpClass(self):
         self.libs = tempfile.mkdtemp()
-        self.repo = "https://github.com/inmanta/"
+        self.repo: str = "https://github.com/inmanta/"
         self.env = tempfile.mkdtemp()
         self.venv = ReentrantVirtualEnv(env_path=self.env)
         config.Config.load_config()
@@ -1068,14 +1134,17 @@ class SnippetCompilationTest(KeepOnFail):
         *,
         autostd: bool = True,
         install_project: bool = True,
-        install_v2_modules: Optional[List[LocalPackagePath]] = None,
-        add_to_module_path: Optional[List[str]] = None,
-        python_package_sources: Optional[List[str]] = None,
-        project_requires: Optional[List[InmantaModuleRequirement]] = None,
-        python_requires: Optional[List[Requirement]] = None,
+        install_v2_modules: Optional[list[LocalPackagePath]] = None,
+        add_to_module_path: Optional[list[str]] = None,
+        python_package_sources: Optional[list[str]] = None,
+        project_requires: Optional[list[InmantaModuleRequirement]] = None,
+        python_requires: Optional[list[Requirement]] = None,
         install_mode: Optional[InstallMode] = None,
-        relation_precedence_rules: Optional[List[RelationPrecedenceRule]] = None,
+        relation_precedence_rules: Optional[list[RelationPrecedenceRule]] = None,
         strict_deps_check: Optional[bool] = None,
+        use_pip_config_file: bool = False,
+        index_url: Optional[str] = None,
+        extra_index_url: list[str] = [],
     ) -> Project:
         """
         Sets up the project to compile a snippet of inmanta DSL. Activates the compiler environment (and patches
@@ -1094,6 +1163,8 @@ class SnippetCompilationTest(KeepOnFail):
         :param relation_precedence_policy: The relation precedence policy that should be stored in the project.yml file of the
                                            Inmanta project.
         :param strict_deps_check: True iff the returned project should have strict dependency checking enabled.
+        :param use_pip_config_file: True iff the pip config file should be used and no source is required for v2 to work
+                                    False if a package source is needed for v2 modules to work
         """
         self.setup_for_snippet_external(
             snippet,
@@ -1103,6 +1174,9 @@ class SnippetCompilationTest(KeepOnFail):
             python_requires,
             install_mode,
             relation_precedence_rules,
+            use_pip_config_file,
+            index_url,
+            extra_index_url,
         )
         return self._load_project(autostd, install_project, install_v2_modules, strict_deps_check=strict_deps_check)
 
@@ -1110,7 +1184,7 @@ class SnippetCompilationTest(KeepOnFail):
         self,
         autostd: bool,
         install_project: bool,
-        install_v2_modules: Optional[List[LocalPackagePath]] = None,
+        install_v2_modules: Optional[list[LocalPackagePath]] = None,
         main_file: str = "main.cf",
         strict_deps_check: Optional[bool] = None,
     ):
@@ -1133,7 +1207,8 @@ class SnippetCompilationTest(KeepOnFail):
         """
         env.mock_process_env(env_path=self.env)
 
-    def _install_v2_modules(self, install_v2_modules: Optional[List[LocalPackagePath]] = None) -> None:
+    def _install_v2_modules(self, install_v2_modules: Optional[list[LocalPackagePath]] = None) -> None:
+        """Assumes we have a project set"""
         install_v2_modules = install_v2_modules if install_v2_modules is not None else []
         module_tool = ModuleTool()
         for mod in install_v2_modules:
@@ -1142,7 +1217,11 @@ class SnippetCompilationTest(KeepOnFail):
                     install_path = mod.path
                 else:
                     install_path = module_tool.build(mod.path, build_dir)
-                self.project.virtualenv.install_from_source(paths=[LocalPackagePath(path=install_path, editable=mod.editable)])
+                self.project.virtualenv.install_for_config(
+                    requirements=[],
+                    paths=[LocalPackagePath(path=install_path, editable=mod.editable)],
+                    config=self.project.metadata.pip,
+                )
 
     def reset(self):
         Project.set(Project(self.project_dir, autostd=Project.get().autostd, venv_path=self.env))
@@ -1152,15 +1231,19 @@ class SnippetCompilationTest(KeepOnFail):
     def setup_for_snippet_external(
         self,
         snippet: str,
-        add_to_module_path: Optional[List[str]] = None,
-        python_package_sources: Optional[List[str]] = None,
-        project_requires: Optional[List[InmantaModuleRequirement]] = None,
-        python_requires: Optional[List[Requirement]] = None,
+        add_to_module_path: Optional[list[str]] = None,
+        python_package_sources: Optional[list[str]] = None,
+        project_requires: Optional[list[InmantaModuleRequirement]] = None,
+        python_requires: Optional[list[Requirement]] = None,
         install_mode: Optional[InstallMode] = None,
-        relation_precedence_rules: Optional[List[RelationPrecedenceRule]] = None,
+        relation_precedence_rules: Optional[list[RelationPrecedenceRule]] = None,
+        use_pip_config_file: bool = False,
+        index_url: Optional[str] = None,
+        extra_index_url: list[str] = [],
     ) -> None:
         add_to_module_path = add_to_module_path if add_to_module_path is not None else []
         python_package_sources = python_package_sources if python_package_sources is not None else []
+
         project_requires = project_requires if project_requires is not None else []
         python_requires = python_requires if python_requires is not None else []
         relation_precedence_rules = relation_precedence_rules if relation_precedence_rules else []
@@ -1175,15 +1258,7 @@ class SnippetCompilationTest(KeepOnFail):
                 - {{type: git, url: {self.repo} }}
             """.rstrip()
             )
-            if python_package_sources:
-                cfg.write(
-                    "".join(
-                        f"""
-                - {{type: package, url: {source} }}
-                        """.rstrip()
-                        for source in python_package_sources
-                    )
-                )
+
             if relation_precedence_rules:
                 cfg.write("\n            relation_precedence_policy:\n")
                 cfg.write("\n".join(f"                - {rule}" for rule in relation_precedence_rules))
@@ -1192,13 +1267,30 @@ class SnippetCompilationTest(KeepOnFail):
                 cfg.write("\n".join(f"                - {req}" for req in project_requires))
             if install_mode:
                 cfg.write(f"\n            install_mode: {install_mode.value}")
+
+            cfg.write(
+                f"""
+            pip:
+                use_system_config: {use_pip_config_file}
+"""
+            )
+            if index_url:
+                cfg.write(
+                    f"""                index_url: {index_url}
+"""
+                )
+            if extra_index_url:
+                cfg.write(
+                    f"""                extra_index_url: [{", ".join(url for url in extra_index_url)}]
+"""
+                )
         with open(os.path.join(self.project_dir, "requirements.txt"), "w", encoding="utf-8") as fd:
             fd.write("\n".join(str(req) for req in python_requires))
         self.main = os.path.join(self.project_dir, "main.cf")
         with open(self.main, "w", encoding="utf-8") as x:
             x.write(snippet)
 
-    def _get_modulepath_for_project_yml_file(self, add_to_module_path: List[str] = []) -> str:
+    def _get_modulepath_for_project_yml_file(self, add_to_module_path: list[str] = []) -> str:
         dirs = [self.libs]
         if self.modules_dir:
             dirs.append(self.modules_dir)
@@ -1211,7 +1303,7 @@ class SnippetCompilationTest(KeepOnFail):
         include_status=False,
         do_raise=True,
         partial_compile: bool = False,
-        resource_sets_to_remove: Optional[List[str]] = None,
+        resource_sets_to_remove: Optional[list[str]] = None,
     ) -> Union[tuple[int, ResourceDict], tuple[int, ResourceDict, dict[str, const.ResourceState]]]:
         return self._do_export(
             deploy=False,
@@ -1231,14 +1323,14 @@ class SnippetCompilationTest(KeepOnFail):
         include_status=False,
         do_raise=True,
         partial_compile: bool = False,
-        resource_sets_to_remove: Optional[List[str]] = None,
+        resource_sets_to_remove: Optional[list[str]] = None,
     ) -> Union[tuple[int, ResourceDict], tuple[int, ResourceDict, dict[str, const.ResourceState]]]:
         """
         helper function to allow actual export to be run on a different thread
         i.e. export.run must run off main thread to allow it to start a new ioloop for run_sync
         """
 
-        class Options(object):
+        class Options:
             pass
 
         options = Options()
@@ -1276,16 +1368,18 @@ class SnippetCompilationTest(KeepOnFail):
         include_status=False,
         do_raise=True,
         partial_compile: bool = False,
-        resource_sets_to_remove: Optional[List[str]] = None,
+        resource_sets_to_remove: Optional[list[str]] = None,
     ) -> Union[tuple[int, ResourceDict], tuple[int, ResourceDict, dict[str, const.ResourceState], Optional[dict[str, object]]]]:
-        return await off_main_thread(
+        """Export to an actual server"""
+        return await asyncio.get_running_loop().run_in_executor(
+            None,
             lambda: self._do_export(
                 deploy=True,
                 include_status=include_status,
                 do_raise=do_raise,
                 partial_compile=partial_compile,
                 resource_sets_to_remove=resource_sets_to_remove,
-            )
+            ),
         )
 
     def setup_for_error(self, snippet, shouldbe, indent_offset=0):
@@ -1371,7 +1465,7 @@ def projects_dir() -> str:
     yield os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
 
 
-class CLI(object):
+class CLI:
     async def run(self, *args, **kwargs):
         # set column width very wide so lines are not wrapped
         os.environ["COLUMNS"] = "1000"
@@ -1394,13 +1488,10 @@ def cli(caplog):
     # due to mysterious interference when juggling with sys.stdout
     # https://github.com/pytest-dev/pytest/issues/10553
     with caplog.at_level(logging.FATAL):
-        # work around for https://github.com/pytest-dev/pytest-asyncio/issues/168
-        asyncio.set_event_loop_policy(AnyThreadEventLoopPolicy())
-        o = CLI()
-        yield o
+        yield CLI()
 
 
-class AsyncCleaner(object):
+class AsyncCleaner:
     def __init__(self):
         self.register = []
 
@@ -1418,7 +1509,7 @@ async def async_finalizer():
     await asyncio.gather(*[item() for item in cleaner.register])
 
 
-class CompileRunnerMock(object):
+class CompileRunnerMock:
     def __init__(
         self, request: data.Compile, make_compile_fail: bool = False, runner_queue: Optional[queue.Queue] = None
     ) -> None:
@@ -1429,7 +1520,7 @@ class CompileRunnerMock(object):
         self._runner_queue = runner_queue
         self.block = False
 
-    async def run(self, force_update: Optional[bool] = False) -> Tuple[bool, None]:
+    async def run(self, force_update: Optional[bool] = False) -> tuple[bool, None]:
         now = datetime.datetime.now()
 
         if self._runner_queue is not None:
@@ -1484,7 +1575,7 @@ async def mocked_compiler_service_block(server, monkeypatch):
 
 
 @pytest.fixture
-def tmpvenv(tmpdir: py.path.local) -> Iterator[Tuple[py.path.local, py.path.local]]:
+def tmpvenv(tmpdir: py.path.local) -> Iterator[tuple[py.path.local, py.path.local]]:
     """
     Creates a venv with the latest pip in `${tmpdir}/.venv` where `${tmpdir}` is the directory returned by the `tmpdir`
     fixture. This venv is completely decoupled from the active development venv.
@@ -1500,8 +1591,8 @@ def tmpvenv(tmpdir: py.path.local) -> Iterator[Tuple[py.path.local, py.path.loca
 
 @pytest.fixture
 def tmpvenv_active(
-    deactive_venv, tmpvenv: Tuple[py.path.local, py.path.local]
-) -> Iterator[Tuple[py.path.local, py.path.local]]:
+    deactive_venv, tmpvenv: tuple[py.path.local, py.path.local]
+) -> Iterator[tuple[py.path.local, py.path.local]]:
     """
     Activates the venv created by the `tmpvenv` fixture within the currently running process. This venv is completely decoupled
     from the active development venv. As a result, any attempts to load new modules from the development venv will fail until
@@ -1597,7 +1688,7 @@ def local_module_package_index(modules_v2_dir: str) -> Iterator[str]:
         if any(not os.path.exists(f) for f in [build_dir, index_dir, timestamp_file]):
             # Cache doesn't exist
             return True
-        if len(os.listdir(index_dir)) != len(os.listdir(modules_v2_dir)) + 1:  # #modules + index.html
+        if len(os.listdir(index_dir)) != len(os.listdir(modules_v2_dir)) + 3:  # #modules + index.html + setuptools + wheel
             # Modules were added/removed from the build_dir
             return True
         # Cache is dirty
@@ -1605,6 +1696,7 @@ def local_module_package_index(modules_v2_dir: str) -> Iterator[str]:
             os.path.getmtime(os.path.join(root, f)) > os.path.getmtime(timestamp_file)
             for root, _, files in os.walk(modules_v2_dir)
             for f in files
+            if "egg-info" not in root  # we write egg info in some test, messing up the tests
         )
 
     if _should_rebuild_cache():
@@ -1617,6 +1709,11 @@ def local_module_package_index(modules_v2_dir: str) -> Iterator[str]:
         for module_dir in os.listdir(modules_v2_dir):
             path: str = os.path.join(modules_v2_dir, module_dir)
             ModuleTool().build(path=path, output_dir=build_dir)
+        # Download bare necessities
+        CommandRunner(logging.getLogger(__name__)).run_command_and_log_output(
+            ["pip", "download", "setuptools", "wheel"], cwd=build_dir
+        )
+
         # Build python package repository
         dir2pi(argv=["dir2pi", build_dir])
         # Update timestamp file
@@ -1640,7 +1737,7 @@ async def migrate_db_from(
     if marker is None or len(marker.args) != 1:
         raise ValueError("Please set the db version to restore using `@pytest.mark.db_restore_dump(<file>)`")
     # restore old version
-    with open(marker.args[0], "r") as fh:
+    with open(marker.args[0]) as fh:
         await PGRestore(fh.readlines(), postgresql_client).run()
 
     bootloader: InmantaBootloader = InmantaBootloader()
@@ -1659,7 +1756,7 @@ async def migrate_db_from(
 @pytest.fixture(scope="session", autouse=not PYTEST_PLUGIN_MODE)
 def guard_invariant_on_v2_modules_in_data_dir(modules_v2_dir: str) -> None:
     """
-    When the test suite runs, the python environment used to build V2 modules is cached using the IsolatedEnvBuilderCached
+    When the test suite runs, the python environment used to build V2 modules is cached using the DefaultIsolatedEnvCached
     class. This cache relies on the fact that all modules in the tests/data/modules_v2 directory use the same build-backand
     and build requirements. This guard verifies whether that assumption is fulfilled and raises an exception if it's not.
     """
@@ -1678,7 +1775,7 @@ All modules present in the tests/data/module_v2 directory should satisfy the abo
 the test suite caches the python environment used to build the V2 modules. This cache relies on the assumption that all modules
 use the same build-backend and build requirements.
         """.strip()
-        with open(pyproject_toml_path, "r", encoding="utf-8") as fh:
+        with open(pyproject_toml_path, encoding="utf-8") as fh:
             pyproject_toml_as_dct = toml.load(fh)
             try:
                 if pyproject_toml_as_dct["build-system"]["build-backend"] != "setuptools.build_meta" or set(
@@ -1746,3 +1843,16 @@ def index_with_pkgs_containing_optional_deps() -> str:
                 publish_index=pip_index,
             )
         yield pip_index.url
+
+
+@pytest.fixture(scope="session", autouse=True)
+def disable_version_and_agent_cleanup_job():
+    """
+    Disable the cleanup job ran by the Inmanta server that cleans up old model version and agent records that are no longer
+    used. Enabling this cleanup for the test suite causes race conditions in tests cases that create agent records without an
+    associated model version.
+    """
+    old_perform_cleanup = orchestrationservice.PERFORM_CLEANUP
+    orchestrationservice.PERFORM_CLEANUP = False
+    yield
+    orchestrationservice.PERFORM_CLEANUP = old_perform_cleanup
