@@ -18,25 +18,28 @@
 import asyncio
 import configparser
 import datetime
+import functools
 import json
 import logging
 import os
 import shutil
 import uuid
 from collections import abc
+from collections.abc import Sequence
 from dataclasses import dataclass
 from datetime import timezone
-from typing import Any, Dict, Optional, Sequence, Type, TypeVar, Union
+from typing import Any, Optional, TypeVar, Union
 
 import pytest
 import yaml
 from pkg_resources import Requirement, parse_version
-from pydantic.tools import lru_cache
 
 import build
 import build.env
 from _pytest.mark import MarkDecorator
 from inmanta import const, data, env, module, util
+from inmanta.data import ResourceIdStr
+from inmanta.data.model import PipConfig
 from inmanta.moduletool import ModuleTool
 from inmanta.protocol import Client
 from inmanta.server.bootloader import InmantaBootloader
@@ -47,8 +50,10 @@ from packaging import version
 
 T = TypeVar("T")
 
+LOGGER = logging.getLogger(__name__)
 
-def get_all_subclasses(cls: Type[T]) -> set[Type[T]]:
+
+def get_all_subclasses(cls: type[T]) -> set[type[T]]:
     """
     Returns all loaded subclasses of any depth for a given class. Includes the class itself.
     """
@@ -102,7 +107,7 @@ def assert_equal_ish(minimal, actual, sortby=[]):
 
             minimal = sorted(minimal, key=keyfunc)
             actual = sorted(actual, key=keyfunc)
-        for (m, a) in zip(minimal, actual):
+        for m, a in zip(minimal, actual):
             assert_equal_ish(m, a, sortby)
     elif minimal is UNKWN:
         return
@@ -111,9 +116,7 @@ def assert_equal_ish(minimal, actual, sortby=[]):
 
 
 def assert_graph(graph, expected):
-    lines = [
-        "%s: %s" % (f.id.get_attribute_value(), t.id.get_attribute_value()) for f in graph.values() for t in f.resource_requires
-    ]
+    lines = [f"{f.id.get_attribute_value()}: {t.id.get_attribute_value()}" for f in graph.values() for t in f.resource_requires]
     lines = sorted(lines)
 
     elines = [x.strip() for x in expected.split("\n")]
@@ -122,7 +125,7 @@ def assert_graph(graph, expected):
     assert elines == lines, (lines, elines)
 
 
-class AsyncClosing(object):
+class AsyncClosing:
     def __init__(self, awaitable):
         self.awaitable = awaitable
 
@@ -186,7 +189,7 @@ def log_index(caplog, loggerpart, level, msg, after=0):
     assert False
 
 
-class LogSequence(object):
+class LogSequence:
     def __init__(self, caplog, index=0, allow_errors=True, ignore=[]):
         """
 
@@ -237,7 +240,7 @@ def assert_no_warning(caplog, loggers_to_allow: list[str] = NOISY_LOGGERS):
     Assert there are no warning, except from the list of loggers to allow
     """
     for record in caplog.records:
-        assert record.levelname != "WARNING" or (record.name in loggers_to_allow)
+        assert record.levelname != "WARNING" or (record.name in loggers_to_allow), record
 
 
 def configure(unused_tcp_port, database_name, database_port):
@@ -270,7 +273,11 @@ async def report_db_index_usage(min_precent=100):
         print(row)
 
 
-async def wait_for_version(client, environment, cnt):
+async def wait_for_version(client, environment, cnt, compile_timeout: int = 30):
+    """
+    :param compile_timeout: Raise an AssertionError if the compilation didn't finish after this amount of seconds.
+    """
+
     # Wait until the server is no longer compiling
     # wait for it to finish
     async def compile_done():
@@ -278,7 +285,7 @@ async def wait_for_version(client, environment, cnt):
         code = compiling.code
         return code == 204
 
-    await retry_limited(compile_done, 30)
+    await retry_limited(compile_done, compile_timeout)
 
     reports = await client.get_reports(environment)
     for report in reports.result["reports"]:
@@ -306,7 +313,20 @@ async def _wait_until_deployment_finishes(client: Client, environment: str, vers
     await retry_limited(is_deployment_finished, timeout)
 
 
-class ClientHelper(object):
+async def _wait_for_resource_actions(
+    client: Client, environment: str, rid: ResourceIdStr, deploy_count: int, timeout: int = 10
+) -> None:
+    async def is_deployment_finished() -> bool:
+        result = await client.resource_logs(environment, rid, filter={"action": ["deploy"]})
+        assert result.code == 200
+        end_lines = [line for line in result.result["data"] if "End run" in line.get("msg", "")]
+        LOGGER.info("Deploys done: %s", end_lines)
+        return len(end_lines) >= deploy_count
+
+    await retry_limited(is_deployment_finished, timeout)
+
+
+class ClientHelper:
     def __init__(self, client: Client, environment: uuid.UUID) -> None:
         self.client = client
         self.environment = environment
@@ -316,7 +336,7 @@ class ClientHelper(object):
         assert res.code == 200
         return res.result["data"]
 
-    async def put_version_simple(self, resources: Dict[str, Any], version: int) -> None:
+    async def put_version_simple(self, resources: dict[str, Any], version: int) -> None:
         res = await self.client.put_version(
             tid=self.environment,
             version=version,
@@ -328,7 +348,7 @@ class ClientHelper(object):
         assert res.code == 200, res.result
 
 
-def get_resource(version: int, key: str = "key1", agent: str = "agent1", value: str = "value1") -> Dict[str, Any]:
+def get_resource(version: int, key: str = "key1", agent: str = "agent1", value: str = "value1") -> dict[str, Any]:
     return {
         "key": key,
         "value": value,
@@ -339,7 +359,7 @@ def get_resource(version: int, key: str = "key1", agent: str = "agent1", value:
     }
 
 
-@lru_cache(1)
+@functools.lru_cache(1)
 def get_product_meta_data() -> ProductMetadata:
     """Get the produce meta-data"""
     bootloader = InmantaBootloader()
@@ -384,7 +404,7 @@ def create_python_package(
     install: bool = False,
     editable: bool = False,
     publish_index: Optional[PipIndex] = None,
-    optional_dependencies: Optional[Dict[str, Sequence[Requirement]]] = None,
+    optional_dependencies: Optional[dict[str, Sequence[Requirement]]] = None,
 ) -> None:
     """
     Creates an empty Python package.
@@ -449,12 +469,14 @@ author = Inmanta <code@inmanta.com>
                 fd.write(f"\n{option_name} ={requirements_as_string}")
 
     if install:
-        env.process_env.install_from_source([env.LocalPackagePath(path=path, editable=editable)])
+        env.process_env.install_for_config(
+            requirements=[],
+            paths=[env.LocalPackagePath(path=path, editable=editable)],
+            config=PipConfig(use_system_config=True),
+        )
     if publish_index is not None:
-        with build.env.IsolatedEnvBuilder() as build_env:
-            builder = build.ProjectBuilder(
-                srcdir=path, python_executable=build_env.executable, scripts_dir=build_env.scripts_dir
-            )
+        with build.env.DefaultIsolatedEnv() as build_env:
+            builder = build.ProjectBuilder(source_dir=path, python_executable=build_env.python_executable)
             build_env.install(builder.build_system_requires)
             build_env.install(builder.get_requires_for_build(distribution="wheel"))
             builder.build(distribution="wheel", output_directory=publish_index.artifact_dir)
@@ -522,7 +544,7 @@ def module_from_template(
         config["metadata"]["name"] = module.ModuleV2Source.get_package_name_for(new_name)
         manifest_file: str = os.path.join(dest_dir, "MANIFEST.in")
         manifest_content: str
-        with open(manifest_file, "r") as fd:
+        with open(manifest_file) as fd:
             manifest_content: str = fd.read()
         with open(manifest_file, "w", encoding="utf-8") as fd:
             fd.write(manifest_content.replace(f"inmanta_plugins/{old_name}/", f"inmanta_plugins/{new_name}/"))
@@ -550,11 +572,23 @@ def module_from_template(
     with open(config_file, "w") as fh:
         config.write(fh)
     if install:
-        ModuleTool().install(editable=editable, path=dest_dir)
+        if editable:
+            env.process_env.install_for_config(
+                requirements=[],
+                paths=[env.LocalPackagePath(path=dest_dir, editable=True)],
+                config=PipConfig(use_system_config=True),
+            )
+        else:
+            mod_artifact_path = ModuleTool().build(path=dest_dir)
+            env.process_env.install_for_config(
+                requirements=[],
+                paths=[env.LocalPackagePath(path=mod_artifact_path)],
+                config=PipConfig(use_system_config=True),
+            )
     if publish_index is not None:
         ModuleTool().build(path=dest_dir, output_dir=publish_index.artifact_dir)
         publish_index.publish()
-    with open(config_file, "r") as fh:
+    with open(config_file) as fh:
         return module.ModuleV2Metadata.parse(fh)
 
 
@@ -581,8 +615,8 @@ def v1_module_from_template(
     """
     shutil.copytree(source_dir, dest_dir)
     config_file: str = os.path.join(dest_dir, module.ModuleV1.MODULE_FILE)
-    config: Dict[str, object] = configparser.ConfigParser()
-    with open(config_file, "r") as fd:
+    config: dict[str, object] = configparser.ConfigParser()
+    with open(config_file) as fd:
         config = yaml.safe_load(fd)
     if new_version is not None:
         config["version"] = str(new_version)
@@ -608,7 +642,7 @@ def v1_module_from_template(
                     for req in new_requirements
                 )
             )
-    with open(config_file, "r") as fd:
+    with open(config_file) as fd:
         return module.ModuleV1Metadata.parse(fd)
 
 
